import { RedisClient, Callback } from 'redis';
import { CacheClient } from '../interfaces';
export declare class RedisAdapter implements CacheClient {
    static buildSetArgumentsFromObject: (objectValue: any) => string[];
    static responseCallback: (resolve: Function, reject: Function) => Callback<any>;
    private redisClient;
    private clientReady;
    private isPingingClient;
    constructor(redisClient: RedisClient);
    /**
     * checkIfReady will return the last received ready status of the client.
     * If the client isn't ready, it will ping the redis client to check if it's ready
     * yet. This isn't a perfect solution, because we're not waiting for the response (so as to
     * not add latency to the underlying method calls). I believe it's a reasonable trade-off to
     * have a potential cache miss rather than add latency to all decorated method calls.
     */
    private checkIfReady;
    getClientTTL(): number;
    get(cacheKey: string): Promise<any>;
    /**
     * set - Sets a key equal to a value in a Redis cache
     *
     * @param cacheKey The key to store the value under
     * @param value    The value to store
     * @param ttl      Time to Live (how long, in seconds, the value should be cached)
     *
     * @returns {Promise}
     */
    set(cacheKey: string, value: any, ttl?: number): Promise<any>;
    del(cacheKey: string): Promise<any>;
}
