"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
const index_1 = require("../index");
/**
 * CacheClear - This decorator allows you to clear a key in
 *
 * @param options {CacheOptions}
 */
function CacheClear(options) {
    return (target, propertyKey, descriptor) => {
        return Object.assign(Object.assign({}, descriptor), { value: function (...args) {
                return __awaiter(this, void 0, void 0, function* () {
                    // Allow a client to be passed in directly for granularity, else use the connected
                    // client from the main CacheManager singleton.
                    const client = options && options.client
                        ? options.client
                        : index_1.default.client;
                    // If there is no client, no-op is enabled (else we would have thrown before),
                    // just return the result of the decorated method (no caching)
                    if (!client) {
                        if (options && options.noop && util_1.determineOp(options.noop, args, this)) {
                            return descriptor.value.apply(this, args);
                        }
                        // A caching client must exist if not set to noop, otherwise this library is doing nothing.
                        if (index_1.default.options.debug) {
                            console.warn('type-cacheable @CacheClear was not set up with a caching client. Without a client, type-cacheable is not serving a purpose.');
                        }
                        return descriptor.value.apply(this, args);
                    }
                    const contextToUse = !index_1.default.options.excludeContext
                        ? this
                        : undefined;
                    const finalKey = util_1.getFinalKey(options && options.cacheKey, options && options.hashKey, propertyKey, args, contextToUse);
                    // Run the decorated method
                    const result = yield descriptor.value.apply(this, args);
                    // Delete the requested value from cache
                    try {
                        yield client.del(finalKey);
                    }
                    catch (err) {
                        if (index_1.default.options.debug) {
                            console.warn(`type-cacheable CacheClear failure due to client error: ${err.message}`);
                        }
                    }
                    return result;
                });
            } });
    };
}
exports.CacheClear = CacheClear;
;
